# -*- coding: utf-8 -*-
#######################################################################

###        ABACOM netPIO /TEMP12 board (Python 2 script)
###        
###      tested on Raspberry Pi Rev.2 (Linux)  ABACOM 11/2013 

###    other linux systems were not tested. Feedback wellcome.

#######################################################################

#######################################################################
### usage: netpio_temp12.py -a <host or IP> -p <UDP port>
#######################################################################

### example calls from linux command line:

###     python ./netpio_temp12.py --address 192.168.1.xxx --port 37157  
### or
###     python ./netpio_temp12.py -a myhostname.somewhere.com -p 37157 


###################### required libraries  #######################

### These libs likely are already pre-installed on RaspPi...
import sys
import time
import getopt
import socket

# Wrap the (USB) request into UDP datagramm
# see netPIO datasheet http://www.abacom-online.de/div/FirmwareDocs/UK/ABACOM_netPIO.pdf
def wrapRequest(aRequest, responseLen, syncByte):
    # add leading "TXDATA" 
    aRequest.insert(0, "T")
    aRequest.insert(1, "X")
    aRequest.insert(2, "D")
    aRequest.insert(3, "A")
    aRequest.insert(4, "T")
    aRequest.insert(5, "A")
    # The USB request is this position
    # add length we exspect (or use syncbyte)
    aRequest.append(responseLen)
    aRequest.append(syncByte)

# execute a netPIO /TEMP12 command string via UDP
def netpioTemp12command(aHost, aPort, aCommand):

    request = bytearray()
    response = bytearray()

    ### build the request...

    Temp12ResponseLen = 226 # TEMP12 response is 226  bytes long
    UnusedSyncByte = 0 # we do not use a sync byte
    
    # command string to byte array
    request = bytearray(aCommand,'ascii') # do someting with device
    # print "Request =", request # if like that
    if len(request)<>21: # check the length, you in case...
        print "Bad request length, must be 21!"

    # Wrap the request into UDP datagramm 
    wrapRequest(request, Temp12ResponseLen, UnusedSyncByte)

    ### ...send the request...

    # get a UDP socket
    UDP = socket.socket(socket.AF_INET, socket.SOCK_DGRAM) # (Internet, UDP)
    # and fire datagramm on UDP
    UDP.sendto(request, (aHost, aPort))


    ### ... and look for the response...

    UDP.settimeout(2.00) # float timeout value in seconds
    try:
        # no need to bind, we are bound by previous send 
        # sock.bind((UDP_IP, UDP_PORT))
        data, addr = UDP.recvfrom(Temp12ResponseLen) # receive from UDP

        # print "Response =", data # TEMP12 response is a readable ASCII string
        # or do whatever you like with it...
        # format to list
        niceList(data);
        
    except socket.timeout:
        print "No response from ",aHost," (",aPort,")"
        sys.exit()

def channelID(channelNo):
    if channelNo in range(0,6):
        return 'B'+str(channelNo)+' '    
    if channelNo in range(6,12):
        return 'C'+str(channelNo-6)+' '    
    if channelNo==12:
        return 'DF1'    
    if channelNo==13:
        return 'DF2'    
    if channelNo==14:
        return 'DF3'    
    if channelNo==15:
        return 'DF4'    
    return '???'

def channelName(channelNo, dataString):
    return unreplaceChars(dataString[(channelNo*14) : (channelNo*14)+6]) 

def channelValue(channelNo, dataString):
    return unreplaceChars(dataString[(channelNo*14)+7 : (channelNo*14)+6+7])

# un-replace LCD char substitutions made by temp12 firmware  
def unreplaceChars(aString):
    result=aString
    result = result.replace(chr(0x01), '')
    result = result.replace(chr(0x02), '')
    result = result.replace(chr(0x03), 'C')
    result = result.replace(chr(0x04), 'F')
    result = result.replace(chr(0x05), 'K')
    result = result.replace(chr(0x06), '')
    return result

def niceList(dataString):
   for i in range(0,16):
       print channelID(i) + ' '+channelName(i,dataString)+' = '+channelValue(i,dataString)       
    

### main method processes the command line ...
def main(argv):

    ### Get string args from command line...
    udpHostString= ''
    udpPortString = ''
    try:
        opts, args = getopt.getopt(argv,"ha:p:",["address=","port="])
    except getopt.GetoptError:
        print 'usage: sudo netpio_temp12.py -a <host> -p <port>'
        sys.exit(2)
    for opt, arg in opts:
        if opt == '-h':
            print 'usage: sudo netpio_temp12.py -a <host> -p <port>'
            sys.exit()
        elif opt in ("-a", "--address"):
            udpHostString = arg
        elif opt in ("-p", "--port"):
            udpPortString = arg
    ### We got the args now, so we can try to use it....

    ### Check UDP host from command line...
    if udpHostString=='': # no host specified in command line
        print 'No host specified!' 
        print 'usage: sudo netpio_temp12.py -a <host> -p <port>'
        sys.exit()

    ### Now get UDP port from command line...
    if udpPortString=='': # no port specified in command line
        print 'No port specified, using default port 37155.' 
        udpPort = 37155
    else:
        try:
            udpPort = int(udpPortString)
        except ValueError:
            print 'Invalid port value!'
            sys.exit()

    if not (udpPort in range(1,65534)):
        print 'Port value out of range (must be 1..65534)'
        sys.exit()

    ### Hurray! We have host and port value.
    ### Here we go...
    # see TEMP12 firmware docs for command details
    # http://www.abacom-online.de/div/FirmwareDocs/UK/Firmware_TEMP12.pdf

    netpioTemp12command(udpHostString, udpPort, 'Bxxxxxxxxxxxx--------' ) # we use "JUST READ" here

    ### finish!  
          
##############################################################
### call the main method with arguments from command line ...
if __name__ == "__main__":
   main(sys.argv[1:])
